@@ -13,6 +13,35 @@ from pygments.lexers import ScalaLexer
 from pygments.formatters import HtmlFormatter
 
 
+class PromptCache:
+
+    def __init__(self):
+        self.clear()
+
+    def clear(self, *ignored):
+        # ignored parameters allow this method to be used as event handler
+        self.next_index = 1
+        self.prompts = {}
+
+    def register_prompt(self, prompt):
+        if prompt in self.prompts:
+            return ''
+        else:
+            index = self.next_index
+            self.next_index = index + 1
+            self.prompts[prompt] = index
+            return """span.prompt{0:d}:before {{
+  content: "{1!s} ";
+}}
+""".format(index, prompt)
+
+    def get_prompt_class(self, prompt):
+        return 'prompt{0:d}'.format(self.prompts[prompt])
+
+
+cache = PromptCache()
+
+
 class PromptDirective(rst.Directive):
 
     optional_arguments = 3
@@ -36,21 +65,16 @@ class PromptDirective(rst.Directive):
             if 'auto' in modifiers:
                 prompts = prompt.split(',')
 
-        html = '<pre class="highlight">'
-        html += '<style type="text/css" scoped>'
+        html = '<div class="highlight-default notranslate"><div class="highlight"><pre>'
+        styles = ''
         if 'auto' in modifiers:
-            for index, prompt in enumerate(prompts):
-                html += """span.prompt%i:before {
-  content: "%s ";
-}
-""" % (index, prompt)
+            for prompt in prompts:
+                styles += cache.register_prompt(prompt)
         else:
             if prompt is not None:
-                html += """span.prompt:before {
-  content: "%s ";
-}
-""" % (prompt)
-        html += '</style>'
+                styles += cache.register_prompt(prompt)
+        if styles:
+            html += '<style type="text/css">\n' + styles + '</style>'
         latex = "\\begin{Verbatim}[commandchars=\\\\\\{\\}]"
 
         Lexer = TextLexer
@@ -62,16 +86,16 @@ class PromptDirective(rst.Directive):
             Lexer = ScalaLexer
 
         statement = []
-        prompt_index = -1
-        for line in self.content:
-            if 'auto' in modifiers:
+        if 'auto' in modifiers:
+            prompt_class = ''
+            for line in self.content:
                 latex += '\n' + line
 
-                for index, prompt in enumerate(prompts):
+                for prompt in prompts:
                     if line.startswith(prompt):
                         if len(statement) > 0:
-                            html += '<span class="prompt%i">%s</span>\n' % (
-                                prompt_index,
+                            html += '<span class="{0!s}">{1!s}</span>\n'.format(
+                                prompt_class,
                                 highlight(
                                     '\n'.join(statement),
                                     Lexer(),
@@ -79,15 +103,27 @@ class PromptDirective(rst.Directive):
                                 ).strip('\r\n')
                             )
                             statement = []
-                        line = line[len(prompt):].strip()
-                        prompt_index = index
+                        line = line[len(prompt):].rstrip()
+                        prompt_class = cache.get_prompt_class(prompt)
                         break
 
                 statement.append(line)
-            elif language == 'bash' or language == 'python':
+            # Add last prompt
+            if len(statement) > 0:
+                html += '<span class="{0!s}">{1!s}</span>\n'.format(
+                    prompt_class,
+                    highlight(
+                        '\n'.join(statement),
+                        Lexer(),
+                        HtmlFormatter(nowrap=True)
+                    ).strip('\r\n')
+                )
+        elif language in ['bash', 'python']:
+            for line in self.content:
                 statement.append(line)
                 if len(line) == 0 or not line[-1] == '\\':
-                    html += '<span class="prompt">%s</span>\n' % (
+                    html += '<span class="{0!s}">{1!s}</span>\n'.format(
+                        cache.get_prompt_class(prompt),
                         highlight(
                             '\n'.join(statement),
                             Lexer(),
@@ -95,12 +131,14 @@ class PromptDirective(rst.Directive):
                         ).strip('\r\n')
                     )
                     if prompt is not None:
-                        latex += '\n%s %s' % (prompt, '\n'.join(statement))
+                        latex += '\n{0!s} {1!s}'.format(prompt, '\n'.join(statement))
                     else:
                         latex += '\n' + '\n'.join(statement)
                     statement = []
-            else:
-                html += '<span class="prompt">%s</span>\n' % (
+        else:
+            for line in self.content:
+                html += '<span class="{0!s}">{1!s}</span>\n'.format(
+                    cache.get_prompt_class(prompt),
                     highlight(
                         line,
                         Lexer(),
@@ -108,22 +146,11 @@ class PromptDirective(rst.Directive):
                     ).strip('\r\n')
                 )
                 if prompt is not None:
-                    latex += '\n%s %s' % (prompt, line)
+                    latex += '\n{0!s} {1!s}'.format(prompt, line)
                 else:
                     latex += '\n' + line
 
-        # Add last prompt
-        if 'auto' in modifiers and len(statement) > 0:
-            html += '<span class="prompt%i">%s</span>\n' % (
-                prompt_index,
-                highlight(
-                    '\n'.join(statement),
-                    Lexer(),
-                    HtmlFormatter(nowrap=True)
-                ).strip('\r\n')
-            )
-
-        html += "</pre>"
+        html += "</pre></div></div>"
         latex += "\n\\end{Verbatim}"
 
         return [
@@ -134,3 +161,8 @@ class PromptDirective(rst.Directive):
 
 def setup(app):
     app.add_directive('prompt', PromptDirective)
+    app.connect('env-purge-doc', cache.clear)
+    return {
+        'parallel_read_safe': True,
+        'parallel_write_safe': True,
+    }
